@@ -21,7 +21,7 @@ from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-__VERSION__ = '0.3.4'
+__VERSION__ = '0.3.6'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -135,7 +135,7 @@ class LoreMailbox:
             return self.msgid_map[msgid]
         return None
 
-    def get_series(self, revision=None):
+    def get_series(self, revision=None, sloppytrailers=False):
         if revision is None:
             if not len(self.series):
                 return None
@@ -201,7 +201,9 @@ class LoreMailbox:
             else:
                 pmsg = self.msgid_map[fmsg.in_reply_to]
 
-            trailers = set(fmsg.trailers)
+            trailers, mismatches = fmsg.get_trailers(sloppy=sloppytrailers)
+            for tname, tvalue in mismatches:
+                lser.trailer_mismatches.add((tvalue, fmsg.fromname, fmsg.fromemail))
             lvl = 1
             while True:
                 logger.debug('%sParent: %s', ' ' * lvl, pmsg.full_subject)
@@ -262,6 +264,10 @@ class LoreMailbox:
                         # Yes, this is very likely our cover letter
                         logger.debug('  fixed revision to v%s', irt.revision)
                         lmsg.revision = irt.revision
+                    # alternatively, see if upthread is patch 1
+                    elif lmsg.counter > 0 and irt is not None and irt.has_diff and irt.counter == 1:
+                        logger.debug('  fixed revision to v%s', irt.revision)
+                        lmsg.revision = irt.revision
 
             # Run our check again
             if lmsg.revision not in self.series:
@@ -293,6 +299,7 @@ class LoreSeries:
         self.expected = expected
         self.patches = [None] * (expected+1)
         self.followups = list()
+        self.trailer_mismatches = set()
         self.complete = False
         self.has_cover = False
 
@@ -569,6 +576,12 @@ class LoreMessage:
             if diffre.search(payload):
                 self.body = payload
 
+        if self.body is None:
+            # Woah, we didn't find any usable parts
+            logger.debug('  No plain or patch parts found in message')
+            logger.info('  Not plaintext: %s', self.full_subject)
+            return
+
         if diffstatre.search(self.body):
             self.has_diffstat = True
         if diffre.search(self.body):
@@ -584,48 +597,64 @@ class LoreMessage:
 
             # Do we have something that looks like a person-trailer?
             matches = re.findall(r'^\s*([\w-]+):[ \t]+(.*<\S+>)\s*$', self.body, re.MULTILINE)
+            # These are commonly part of patch/commit metadata
+            badtrailers = ('from', 'author', 'email')
             if matches:
-                # Basic sanity checking -- the trailer must match the name or the email
-                # in the From header, to avoid false-positive trailer parsing errors
                 for tname, tvalue in matches:
-                    tmatch = False
-                    namedata = email.utils.getaddresses([tvalue])[0]
-                    tfrom = re.sub(r'\+[^@]+@', '@', namedata[1].lower())
-                    hfrom = re.sub(r'\+[^@]+@', '@', self.fromemail.lower())
-                    tlname = namedata[0].lower()
-                    hlname = self.fromname.lower()
-                    tchunks = tfrom.split('@')
-                    hchunks = hfrom.split('@')
-                    if tfrom == hfrom:
-                        logger.debug('  trailer exact email match')
-                        tmatch = True
-                    # See if domain part of one of the addresses is a subset of the other one,
-                    # which should match cases like @linux.intel.com and @intel.com
-                    elif (len(tchunks) == 2 and len(hchunks) == 2
-                          and tchunks[0] == hchunks[0]
-                          and (tchunks[1].find(hchunks[1]) >= 0 or hchunks[1].find(tchunks[1]) >= 0)):
-                        logger.debug('  trailer fuzzy email match')
-                        tmatch = True
-                    # Does the name match, at least?
-                    elif tlname == hlname:
-                        logger.debug('  trailer exact name match')
-                        tmatch = True
-                    # Finally, see if the header From has a comma in it and try to find all
-                    # parts in the trailer name
-                    elif hlname.find(',') > 0:
-                        nmatch = True
-                        for nchunk in hlname.split(','):
-                            if hlname.find(nchunk.strip()) < 0:
-                                nmatch = False
-                                break
-                        if nmatch:
-                            logger.debug('  trailer fuzzy name match')
-                            tmatch = True
-                    if tmatch:
+                    if tname.lower() not in badtrailers:
                         self.trailers.add((tname, tvalue))
-                    else:
-                        logger.debug('  ignoring "%s: %s" due to from mismatch (from: %s %s)', tname, tvalue,
-                                     self.fromname, self.fromemail)
+
+    def get_trailers(self, sloppy=False):
+        mismatches = set()
+        if sloppy:
+            return set(self.trailers), mismatches
+
+        trailers = set()
+        for tname, tvalue in self.trailers:
+            # Don't do from-matching on certain trailers like Fixes:
+            if tname.lower() in ('fixes',):
+                trailers.add((tname, tvalue))
+                continue
+
+            tmatch = False
+            namedata = email.utils.getaddresses([tvalue])[0]
+            tfrom = re.sub(r'\+[^@]+@', '@', namedata[1].lower())
+            hfrom = re.sub(r'\+[^@]+@', '@', self.fromemail.lower())
+            tlname = namedata[0].lower()
+            hlname = self.fromname.lower()
+            tchunks = tfrom.split('@')
+            hchunks = hfrom.split('@')
+            if tfrom == hfrom:
+                logger.debug('  trailer exact email match')
+                tmatch = True
+            # See if domain part of one of the addresses is a subset of the other one,
+            # which should match cases like @linux.intel.com and @intel.com
+            elif (len(tchunks) == 2 and len(hchunks) == 2
+                  and tchunks[0] == hchunks[0]
+                  and (tchunks[1].find(hchunks[1]) >= 0 or hchunks[1].find(tchunks[1]) >= 0)):
+                logger.debug('  trailer fuzzy email match')
+                tmatch = True
+            # Does the name match, at least?
+            elif tlname == hlname:
+                logger.debug('  trailer exact name match')
+                tmatch = True
+            # Finally, see if the header From has a comma in it and try to find all
+            # parts in the trailer name
+            elif hlname.find(',') > 0:
+                nmatch = True
+                for nchunk in hlname.split(','):
+                    if hlname.find(nchunk.strip()) < 0:
+                        nmatch = False
+                        break
+                if nmatch:
+                    logger.debug('  trailer fuzzy name match')
+                    tmatch = True
+            if tmatch:
+                trailers.add((tname, tvalue))
+            else:
+                mismatches.add((tname, tvalue))
+
+        return trailers, mismatches
 
     def __repr__(self):
         out = list()
@@ -658,8 +687,7 @@ class LoreMessage:
 
     @staticmethod
     def clean_header(hdrval):
-        uval = hdrval.replace('\n', ' ')
-        new_hdrval = re.sub(r'\s+', ' ', uval)
+        new_hdrval = re.sub(r'\n?\s+', ' ', str(hdrval))
         return new_hdrval.strip()
 
     @staticmethod
@@ -783,8 +811,7 @@ class LoreMessage:
 
                 # Now we add mix-in trailers
                 btrailers.reverse()
-                trailers = set(self.followup_trailers)
-                trailers.update(set(btrailers))
+                trailers = btrailers + list(self.followup_trailers)
                 added = list()
                 if trailer_order is None:
                     trailer_order = DEFAULT_TRAILER_ORDER
